indexing
	description: "Objects that ..."
	author: ""
	date: "$Date$"
	revision: "$Revision$"

class
	FRACTION1	

inherit
	NUMERIC
		redefine
			is_equal,
			out
		end

create
	make

feature -- Initialization
	make (a_numerator, a_denumerator: INTEGER) is
			-- Creates a fraction
	require
		no_division_by_zero: a_denumerator /= 0
	do
		numerator := a_numerator * a_denumerator.sign
		denumerator := a_denumerator.abs
		reduce
	end

feature -- Identity generation	

	one: like Current is
		-- Creates a one fraction
		do
			create Result.make (1, 1)
		end

	zero: like Current is
		-- Creates a zero fraction
		do
			create Result.make (0, 1)
		end

feature -- Comparison

	divisible (other: like Current): BOOLEAN is
		-- Is the fraction divisible by other?
		do
			Result := not zero.is_equal (other)
		end

	exponentiable (other: like Current): BOOLEAN is
		-- Is the fraction exponentiable by other?
		do
			Result := false
		end

	is_equal (other: like Current): BOOLEAN is
			-- Checks if another fraction is equal to myself
		do
			if numerator = 0 then
				Result := numerator = other.numerator
			else
				Result := numerator = other.numerator and denumerator = other.denumerator
			end
		end

feature -- Fraction operations

	infix "+" (other: like Current): like Current is
			-- Plus operation		
		do
			create Result.make(numerator * other.denumerator + other.numerator * denumerator, denumerator * other.denumerator)
		end

	infix "-" (other: like Current): like Current is
			-- Minus operation
		do
			create Result.make(numerator * other.denumerator - other.numerator * denumerator, denumerator * other.denumerator)
		end

	infix "*" (other: like Current): like Current is
			-- Multiply operation
		do
			create Result.make (numerator * other.numerator, denumerator * other.denumerator)
		end

	infix "/" (other: like Current): like Current is
			-- Divide operation
		do
			create Result.make (numerator * other.denumerator, denumerator * other.numerator)
		end

	prefix "+": like Current is
			-- Positive fraction
		do
			create Result.make (numerator, denumerator)
		end

	prefix "-": like Current is
			-- Negative fraction
		do
			create Result.make (numerator * -1, denumerator)
		end

feature -- Output
	out: STRING is
			-- Outputs the fraction
		do
			Result := numerator.out + "/" + denumerator.out
		end

feature -- Attributes
	numerator: INTEGER
	denumerator: INTEGER

feature {NONE} -- Implementation
	gcd (i, j: INTEGER): INTEGER is
			-- Calculate the greatest common divisor of i and j.
		local
			m, n: INTEGER
		do
			if i = 0 then
				Result := j.abs
			elseif j = 0 then
				Result := i.abs
			else
				-- If neither i nor j are equal to 0 calculate the gcd
				from
					m := i.abs
					n := j.abs
				invariant
					m>0
					n>0
					gcd (m,n) = gcd (i,j)
				variant
					m.max (n)
				until
					m = n
				loop
					if
						m > n
					then
						m := m - n
					else
						n := n - m
					end
				end
				Result := m
			end
		end

	reduce is
			-- Reduces the fraction
		local
			g: INTEGER
		do
			g := gcd(numerator, denumerator)
			numerator := numerator // g
			denumerator := denumerator // g
		end

invariant
	no_division_by_zero: denumerator /= 0
end
