indexing
	description: "A date, which can be created with a string of format ??.??.????"
	author: "N.Ranieri"
	date: "$Today$"
	revision: "$1.0$"

class
	DATE2

create
	make

feature -- Initialization

	make( a_date : STRING ) is
		-- creates the date
		local
			day_string : STRING
			month_string : STRING
			year_string : STRING
		do
			--initialize variables
			is_valid := True
			day := -1
			month := -1
			year := -1

			-- date not void
			if a_date = Void then
				is_valid := false
			-- date must contain 10 characters
			elseif a_date.count /= 10 then
				is_valid := false
			-- third and sixth characters are dots
			elseif a_date.item (3) /= '.' or a_date.item (6) /= '.' then
				is_valid := false
			else
				day_string := a_date.substring (1, 2)
				month_string := a_date.substring (4, 5)
				year_string := a_date.substring (7, 10)

				--test for whitespaces or other symbols -> " 1" or "-1"
				if day_string.is_integer and (not day_string.item(1).is_digit or not day_string.item(2).is_digit) then
					is_valid := false
				end
				if month_string.is_integer and (not month_string.item(1).is_digit or not month_string.item(2).is_digit) then
					is_valid := false
				end
				if year_string.is_integer and (not year_string.item(1).is_digit or not year_string.item(2).is_digit or not year_string.item(3).is_digit or not year_string.item(4).is_digit) then
					is_valid := false
				end

				-- if still valid, start testing the day/month/year
				if is_valid then
					-- test for valid day, must be in [1,31] or ??
					if not ( day_string.is_equal ("??") or (day_string.is_natural and then day_string.to_integer <= 31 and day_string.to_integer >= 1) ) then
						is_valid := false
					end
					-- test for valid month, must be in [1,12] or ??
					if not ( month_string.is_equal ("??") or (month_string.is_natural and then month_string.to_integer <= 12 and month_string.to_integer >= 1) ) then
						is_valid := false
					end
					-- test for valid year, must be natural or ????
					if not (year_string.is_equal ("????") or year_string.is_natural) then
						is_valid := false
					end

					-- test for valid order of unknowns
					if year_string.is_equal ("????") and not month_string.is_equal ("??") then
						is_valid := false
					end
					if month_string.is_equal ("??") and not day_string.is_equal ("??") then
						is_valid := false
					end
				end
			end

			-- if still valid, convert the string to integers
			if is_valid then
				if day_string.is_integer then
					day := day_string.to_integer
				else
					day := 0
				end

				if month_string.is_integer then
					month := month_string.to_integer
				else
					month := 0
				end

				if year_string.is_integer then
					year := year_string.to_integer
				else
					year := 0
				end

				--set the string representation
				string := a_date
			end

			-- specification level is -1 if invalid, or the number of defined parts (0 -> unknown, 1 -> year known etc.)
			if not is_valid then
				specification_level := -1
			elseif not year_string.is_integer then
				specification_level := 0
			elseif not month_string.is_integer then
				specification_level := 1
			elseif not day_string.is_integer then
				specification_level := 2
			else
				specification_level := 3
			end
		end

feature -- queries

	is_valid : BOOLEAN
		--indicates whether the date is/was in valid format or not

	specification_level : INTEGER
		--indicates how much parts are known or -1 if invalid, and hence in {-1,0,1,2,3}

	is_later_than( another_date : DATE2 ) : BOOLEAN is
		-- returns true, if this date is later than another_date
		-- returns false, if it is unknown, equal or lesser
		-- also returns false if one or both of the dates are invalid
		do
			Result := false
			-- both years comparable, not equals ????
			if year > 0 and another_date.year > 0 then
				-- year is allready older
				if year > another_date.year then
					Result := true
				-- year is equal
				elseif year = another_date.year then
					-- months are comparable
					if month > 0 and another_date.month > 0 then
						-- month is older
						if month > another_date.month then
							Result := true
						-- month is equal
						elseif month = another_date.month then
							-- days are comparable
							if day > 0 and another_date.day > 0 then
								-- day is older
								if day > another_date.day then
									Result := true
								end
							end
						end
					end
				end
			end
		end

	string_representation : STRING is
		-- returns the string or void, when not valid
		do
			if string /= Void then
				Result := string.twin
			end
		end


feature {DATE2} -- private fields

	day : INTEGER		-- equals zero stands for ??
	month : INTEGER		-- equals zero stands for ??
	year : INTEGER		-- equals zero stands for ????
	string : STRING		--the sstring representation, or void if it is/was invalid

invariant
	valid_specification_level: specification_level > -2 and specification_level < 4

end
