indexing
	description: "Objects that execute features of class NODE"
	author: ""
	date: "$Date$"
	revision: "$Revision$"

class
	NODE_TESTER

inherit
	CLASS_TESTER
		undefine
			out
		end

	LINKED_GRAPH [INTEGER, STRING]
		rename
			item as graph_item,
			make_empty_linked_graph as make
		end
create
	make

feature -- Basic operations
	execute_small_suite is
			-- Execute small test suite
		do
			test1 (10)
		end

	execute_medium_suite is
			-- Execute medium test suite
		do
			test1 (20)
			test2 (10)
			test3 (10)
			test4 (10)
		end

	execute_large_suite is
			-- Execute large test suite
		do
			test1 (200)
			test2 (100)
			test3 (100)
			test4 (100)
		end

feature {NONE} -- Parameters
	creation_probability: REAL is 0.3

	label_size: INTEGER is 10

feature {NONE} -- Tests
	test1 (count: INTEGER) is
			-- Test 1 (random items for node and other)
	local
		i: INTEGER
	do
		create node.make (random.item)
		random.forth
		create other.make (random.item)
		random.forth
		from
			i := 1
		until
			i > count
		loop
			execute_functions
			if random.decide (0.5) then
				random.forth
				if random.decide (creation_probability) then
					random.forth
					other.reset
					create other.make (random.item)
				end
				random.forth
				node.set_processed
				node.set_referrer (other, edge (other, node), random.bounded_real_item (node.distance / 2, node.distance))
			else
				random.forth
				if random.decide (creation_probability) then
					random.forth
					node.reset
					create node.make (random.item)
				end
				random.forth
				other.set_referrer (node, edge (node, other), random.bounded_real_item (other.distance / 2, other.distance))
				other.set_processed
			end
			random.forth
			i := i + 1
		end
	end

	test2 (count: INTEGER) is
			-- Test 1 (zero items for node, random items for other)
	local
		i: INTEGER
	do
		create node.make (0)
		create other.make (random.item)
		random.forth
		from
			i := 1
		until
			i > count
		loop
			execute_functions
			if random.decide (0.5) then
				random.forth
				if random.decide (creation_probability) then
					random.forth
					other.reset
					create other.make (random.item)
				end
				random.forth
				node.set_processed
				node.set_referrer (other, edge (other, node), random.bounded_real_item (node.distance / 2, node.distance))
			else
				random.forth
				if random.decide (creation_probability) then
					random.forth
					node.reset
				end
				random.forth
				other.set_referrer (node, edge (node, other), random.bounded_real_item (other.distance / 2, other.distance))
				other.set_processed
			end
			random.forth
			i := i + 1
		end
	end

	test3 (count: INTEGER) is
			-- Test 3 (setting referrer to Void)
	local
		i: INTEGER
	do
		create node.make (random.item)
		random.forth
		from
			i := 1
		until
			i > count // 2
		loop
			create other.make (random.item)
			random.forth
			node.set_referrer (other, edge (other, node), random.bounded_real_item (node.distance / 2, node.distance))
			random.forth
			node.set_referrer (Void, Void, 0)
			execute_functions
			node.reset
			i := i + 1
		end
	end

	test4 (count: INTEGER) is
			-- Test 4 (only node)
	local
		i: INTEGER
	do
		create node.make (random.item)
		random.forth
		from
			i := 1
		until
			i > count
		loop
			other := node
			execute_functions
			if random.decide (creation_probability) then
				random.forth
				node.reset
				create node.make (random.item)
			end
			random.forth
			node.set_processed
			node.set_referrer (node, edge (node, node), random.bounded_real_item (node.distance / 2, node.distance))
			random.forth
			i := i + 1
		end
	end

feature {NONE} -- Implementation
	node, other: NODE [INTEGER, STRING]

	execute_functions is
			-- Execute function on `node' and `other'
		local
			res: INTEGER
			b: BOOLEAN
		do
			res := node.hash_code
			if random.decide (0.5) then
				b := node < other
			else
				b := other < node
			end
			random.forth
		end

	edge (n1, n2: like node): EDGE [INTEGER, STRING] is
			-- An edge connecting `n1' and `n2'
		do
			if random.decide (0.5) then
				create Result.make_directed (n1.item, n2.item, random.alpha_string_or_void_item (label_size))
			else
				create Result.make_undirected (n2.item, n1.item, random.alpha_string_or_void_item (label_size))
			end
			random.forth
		end
end
